
/**********************************************************************************
WelcomeStartup:										

		The WelcomeStartup Application issues an appropriate startup greeting 
		depending on the time of day.  The time periods are defined as:
			Morning	:	5 AM - Noon
			MidDay	:	Noon - 8 PM
			Evening	:	8 PM - 5 AM
		and can not be changed without recompilation. (Future Enhcancement?)
		
		Just doubleclick on the application and sit back, it doesn't take long.
		It requires no other user action.  
		
		The shell and the actual sound playing parts of these programs are derived 
		from the Dialogger and Sound101 applications written and presented in 
		MacTech Magazine.  This is intended to be Freeware.  

Author:
		Stephen R. Gross
		3135 Elmendorf Drive
		Oakton, Va.  22124
		
		Internet:	sgross@mitre.org
		
		Feel free to send email, commments, or postcards.  This will give me an idea
		of how truly global the net really is.  Enjoy
		
**********************************************************************************/
#include "WelcomeStartup.h"

int				gDestroyChannel;
SndChannelPtr	gSndChan;
short			gFRefNum;
short			gAppFRefNum;
DateTimeRec		now;

/**********************************************************************************

Routine:	main

	routine does initialization, 
	calls the PlaySound routine,
	Exits

**********************************************************************************/
void	main(void)

{
InitGraf((Ptr) &thePort);
InitFonts();
InitWindows();
InitMenus();
TEInit();
InitDialogs(0L);
InitCursor();

MaxApplZone();

PlayASound();
Terminate();
}
/************************************************************************************

PlayASound

Play the correct sound resource depending on the time of day.  
The time periods are defined as:
	Morning	:	5 AM - Noon
	MidDay	:	Noon - 8 PM
	Night	:	8 PM - 5 AM
 
*************************************************************************************/
void PlayASound(void)
{
OSErr 		myErr;
Handle		tempHandle;
Handle		mornHandle, midHandle, niteHandle;
int			tempId;
OSType		tempType;
int			sndId	= 0;
Str255		tempStr;
CursHandle	hCurs;
Cursor		waitCursor;
	
/* allocate a new sound channel */
gSndChan = 0;
myErr = SndNewChannel(&gSndChan, sampledSynth, initMono, NULL);
		
hCurs = GetCursor(1300); /* set the cursor to the command-period cursor */
waitCursor = **hCurs;
SetCursor(&waitCursor);
		
//	Select the proper sound to play based on the time of day.
GetTime( &now );
{
if ((now.hour >= 5) && (now.hour < 12))
	{
	sndId = kMornSoundID;
	}
else
	if ((now.hour >= 12) && (now.hour < 20))
		{
		sndId = kMidDaySoundID;
		}
	else
		if ((now.hour >= 20) || (now.hour < 5))
			{
			sndId = kNiteSoundID;
			}
}
tempHandle = Get1Resource('snd ',sndId);
GetResInfo(tempHandle,&tempId,&tempType,tempStr);
ReleaseResource(tempHandle);

/* play the sound */
PlaySND(tempId);
}
/************************************************************************************

PlaySND (from SOUND101 in MacTech Magazine 1993)

Plays an 'snd ' resource sound using the SndStartFilePlay toolbox call.  The only 
parameter is the resource id of the sound to be played.  The sound file has allready 
been opened by the routine that called PlaySND.  We pass the id of the sound resource
to be played to the SndStartFilePlay routine.

The sound is played asynchronously.  For this reason, we set a global variable
(gDestroyChannel) to FALSE.  When the sound is done playing our callback routine is 
called (MyCallBack), which sets gDestroyChannel to TRUE.  This is our indication that
the sound is done playing and we can go ahead and close the file, and free the sound
channel.

NOTE:
The userInfo field of the sound channel structure is a convenient place to store the
current A5.  This way in the callback routine we can have access to our A5, and thus have 
access to our globals.  See the TechNotes for more info about SetCurrentA5 and SetA5.

*************************************************************************************/
void PlaySND(int tempId)
{
	gDestroyChannel = 0x0;
	gSndChan->userInfo = SetCurrentA5();
	/* since this is a sound resource, we pass its id to SndStartFilePlay */
 	SndStartFilePlay(gSndChan, 0, tempId, 20L*1024L, 0x0L,
                             0x0L, (ProcPtr) MyCallBack, FALSE);
//                             0x0L, (ProcPtr) MyCallBack, TRUE);
//	I changed TRUE to FALSE to prevent the application from quitting before the sound 
//	finished playing
}

pascal void MyCallBack (SndChannelPtr chan)
{
	OSErr myErr;
	long  myA5;
	
	myA5 = SetA5( chan->userInfo ); 	
	gDestroyChannel = TRUE;
	myA5 = SetA5( myA5 ); 	

}

/**********************************************************************************

Routine:	Terminate

	Quit the WelcomeStartup Application

**********************************************************************************/
void Terminate(void)
/*   exit if no cancellation   */
{
ExitToShell();
}  /*   Terminate  */

/*********************************************************************************/
