
/*******************************************************************************
	FILE:			WelcomeStartupUtility_play.c

	AUTHOR:			S.R. Gross
	
	Revision History:
	
	950220	-	Finally completed a version good enough to be called V1.0
				The Sound Routines were all extracted or derived from 
				Sound101_recordplay.c MacTech Magazine March 1993.  
				The basic application shell and many of the dialog box
				processing routines were extracted or derived from Dialogger.c 
				published in MacTech Magazine.
				
	941215	-	Dusted off earlier attempts to make this work. Really, this 
				time I mean it!!!
				
	930420	-	Modified Menus and Dialog Boxes and their Constants
				to Fit My needs
				Described the Prototypes 
				
	930301	-	Started with Sound101_recordplay.c Listing Provided in March 93 
				MacTech Magazine  1992 Praxitel, Inc. and Iggi Monahelis
 
	==========================================================
  	==========================================================

   ******************************************************************************/

#include "WelcomeStartup.h"

/************************************/
/*             Globals				*/
/************************************/
short			gCurrentSound	=	kBaseResID;
int 			gDestroyChannel =	FALSE;
SndChannelPtr	gSndChan;
short 			gFRefNum, gAppFRefNum;
DateTimeRec		now;

/************************************************************************************

PlayASound

Play the correct sound resource depending on the time of day.  
The time periods are defined as:
	Morning	:	5 AM - Noon
	MidDay	:	Noon - 8 PM
	Night	:	8 PM - 5 AM
 
*************************************************************************************/
void PlayASound(void)
{
SFReply		reply;
OSErr 		myErr;
Handle		tempHandle;
Handle		mornHandle, midHandle, niteHandle;
int 		numResources;
int			tempId;
OSType		tempType;
int			sndId	= 0;
Str255		tempStr;
CursHandle	hCurs;
Cursor		waitCursor;
	
/* allocate a new sound channel */
gSndChan = 0;
myErr = SndNewChannel(&gSndChan, sampledSynth, initMono, NULL);
		
hCurs = GetCursor(1300); /* set the cursor to the command-period cursor */
waitCursor = **hCurs;
SetCursor(&waitCursor);
		
{
GetTime( &now );
fprintf(stdout, "today is %d/%d/%d at %d:%d:%d\n", now.month, now.day, now.year,
					now.hour, now.minute, now.second) ;
//	Select the proper sound to play based on the time of day.
{
if ((now.hour >= 5) && (now.hour < 12))
	{
	sndId = kMornSoundID;
	}
else
	if ((now.hour >= 12) && (now.hour < 20))
		{
		sndId = kMidDaySoundID;
		}
	else
		if ((now.hour >= 20) || (now.hour < 5))
			{
			sndId = kNiteSoundID;
			}
}
//fprintf(stdout, "sndId=%d\n", sndId) ;
UseResFile(gAppFRefNum);
tempHandle = Get1Resource('snd ',sndId);
GetResInfo(tempHandle,&tempId,&tempType,tempStr);
ReleaseResource(tempHandle);

/* play the sound */
//fprintf(stdout, "tempId=%d\n", tempId) ;
PlaySND(tempId);
}
}

/************************************************************************************

PickASound

Prompts the user to select a sound, Morning, Afternoon or Evening, to replace.
Presents the standard SFGetFile dialog to allow the user to select the
replacement sound
Plays the replacement sound as confirmation to the user.
 
*************************************************************************************/

void PickASound(void)
{

SFReply			reply;
OSErr 			myResErr, myErr;
ResType			oldType, newType;
Point 			myPoint;
int				sndId, oldId, newId;
Handle			oldHandle, newHandle;
short			oldResAttrs, newResAttrs;
Str255			oldStr, newStr;
CursHandle		hCurs;
Cursor			waitCursor;


if (gAppFRefNum == 0)
	{
	fprintf(stdout, "Please locate WelcomeStartup application first with the \n");
	fprintf(stdout, "LocateApp menu item.  \n");
	}
else
	{

//Decide which sound to replace 	

	sndId	=	DoDialog();

//fprintf(stdout, "sndId=%d\n", sndId) ;

/* Select Id of sound resource to replace from Application	*/
	
	if (sndId != 0)
		{
		UseResFile(gAppFRefNum);
		strcpy(&oldType, "snd ") ;
		oldHandle = Get1Resource(oldType,sndId);
		GetResInfo(oldHandle,&sndId,&oldType,oldStr);
		}	
	
/* Prepare Dialog Box for user to select a resource file with SFGetFile 
	to select a sound from */

	gFRefNum = 0;
	
	/* let user select the sound file to play */
	if ( (sndId != 0) &&( DisplayGetFile( &reply )) ) 
		{			
  		
/* Get a resource file for the sound and open it 	*/

		SetVol(0L,reply.vRefNum);

		gFRefNum = OpenResFile(reply.fName);
		UseResFile(gFRefNum);
		newHandle = Get1IndResource('snd ',1);
		GetResInfo(newHandle,&newId,&newType,newStr);
		newResAttrs	=	GetResAttrs(newHandle);
		DetachResource(newHandle) ;
	
		UseResFile(gAppFRefNum);
		myResErr	=	0;

		while (! myResErr )
			{
			RmveResource(oldHandle);
			myResErr = ResError();
			}

		switch ( myResErr )
			{
			case rmvResFailed:
//		No more 'snd ' resources of id # sndId
				SetResAttrs(newHandle, newResAttrs);				
				SetResInfo(newHandle, sndId, newStr);				
				AddResource(newHandle, newType, sndId, newStr);
				ChangedResource(newHandle);
				WriteResource(newHandle);
				UpdateResFile(gAppFRefNum);
				FlushVol( 0L, gAppFRefNum );
				CloseResFile(gFRefNum);
				FlushVol( 0L, gFRefNum );
//				fprintf(stdout, "Sound Resource Successfully Replaced ID\n") 

//				fprintf(stdout, "Sound Resource Successfully Replaced ID#%d myResErr=%d\n", 
//						myResErr);

// allocate a new sound channel
				gSndChan = 0;
				myErr = SndNewChannel(&gSndChan, sampledSynth, initMono, NULL);
		
				hCurs = GetCursor(1300); /* set the cursor to the command-period cursor */
				waitCursor = **hCurs;
				SetCursor(&waitCursor);

//	Play the sound 
				PlaySND(sndId);
				break;

				case noErr:
					fprintf(stdout, "Multiple Sound Resources Successfully Removed ID#%d myResErr=%d\n", 
						myResErr);
				break;

				default:

//					Unexpected ResError()
					GetTime( &now );
					fprintf(stdout, "today is %d/%d/%d at %d:%d:%d\n", now.month, now.day, now.year,
						now.hour, now.minute, now.second) ;
					fprintf(stdout, "Unexpected ResError() %d\n", myResErr);
					break;
				}
	
//	SetPort(thePort);
			}
 	else
		{
		GetTime( &now );
		fprintf(stdout, "today is %d/%d/%d at %d:%d:%d\n", now.month, now.day, now.year,
						now.hour, now.minute, now.second) ;
		fprintf(stdout, "No Changes made to the Sound Resources.\n");
		}
	}
}
/************************************************************************************

LocateApp

Prompts the user with the standard SFGetFile dialog to locate and select the 
WelcomeStartup application (where the sounds are stored) to use for this run.   
 
*************************************************************************************/

void LocateApp(void)
{
SFReply			*appReply;
OSErr 			myResErr, myErr;
ResType			myTypeList[4];
Point			myPoint = {50,50};

//	Prepare SFGetFile Dialog for user to select the WelcomeStartup application 

	gAppFRefNum = 0;
	
	strcpy(&myTypeList[0], "APPL") ;
		
	SFGetFile(myPoint,"\pFind WelcomeStartup Application",NULL,3,myTypeList,NULL,appReply);

	if ( appReply->good ) 
		{			
		SetVol(0L,appReply->vRefNum);

		gAppFRefNum = OpenResFile(appReply->fName);
		}
}
/*************************************************************************************/
/************************************/
/*       		DoDialog			*/
/************************************/

int		DoDialog( void )
{

DialogPtr	myDialog;
Boolean		dialogDone	=	false;
short		itemHit,	itemType;
Handle		itemHandle;
Rect		itemRect;
int			whichSndId = 0;
short		curRadioButton;

myDialog	=	GetNewDialog( kDialogResID, nil, kMoveToFront );

/*if (myDialog)
	{
	*/
	ShowWindow(myDialog );
	SelectWindow(myDialog);
	SetPort( myDialog );

	SetDialogDefaultItem( myDialog, iDone );
	SetDialogCancelItem( myDialog, iCancel);

	whichSndId	=	kMornSoundID;
	curRadioButton 	=	kFirstRadio;
	GetDItem( myDialog, curRadioButton, &itemType, &itemHandle, &itemRect );
	SetCtlValue( (ControlHandle)itemHandle, kOn );

while ( ! dialogDone )
	{
	ModalDialog( nil, &itemHit );

	switch ( itemHit )
		{
		case iDone:
			dialogDone	=	true;
			break;
		case iCancel:
			dialogDone	=	true;
			whichSndId	=	0;
			break;
		case iMorning:
			whichSndId	=	kMornSoundID;
		case iMidDay:
			if (itemHit == iMidDay)
			whichSndId	=	kMidDaySoundID;
		case iNite:
			if (itemHit == iNite)
				whichSndId	=	kNiteSoundID;
			if ( curRadioButton != itemHit)
				{
				GetDItem( myDialog, curRadioButton, &itemType, &itemHandle, &itemRect );
				FlipControl( (ControlHandle)itemHandle );

				GetDItem( myDialog, itemHit, &itemType, &itemHandle, &itemRect );
				FlipControl( (ControlHandle)itemHandle );

				curRadioButton	=	itemHit;
				}
			break;
		}	
	}		

HideWindow( myDialog);

DisposDialog( myDialog );
//fprintf(stdout, "whichSndId=%d\n", whichSndId) ;
return whichSndId ;
}
/************************************/
/*       	FlipControl				*/
/************************************/

void		FlipControl( ControlHandle control )
{
SetCtlValue( control, ! GetCtlValue( control ) );
}

/************************************************************************************

DisplayGetFile

Prompts the user to select a file thru the standard file
dialog.  
 
*************************************************************************************/
Boolean DisplayGetFile(SFReply *reply)
{
ResType			myTypeList[4];
Point			myPoint = {50,50};

	strcpy(&myTypeList[0], "sfil") ;
	strcpy(&myTypeList[1], "rsrc") ;
	strcpy(&myTypeList[2], "snd ") ;
		
	SFGetFile(myPoint,"\pPick a sound from:",NULL,3,myTypeList,NULL,reply);

	return(reply->good);
}

/************************************************************************************

ToggleRadioButtons
SetRadioButton

Couple of utility routines to set and toggle radio buttons.
 
*************************************************************************************/
void	ToggleRadioButtons( DialogPtr theDialog, short oldItem, short newItem)
{
	ControlHandle	control;
	short			itemType;
	Rect			box;
	
	GetDItem(theDialog, oldItem,  &itemType, &control, &box);
	SetCtlValue(control, 0);
	GetDItem(theDialog, newItem, &itemType, &control, &box);
	SetCtlValue(control, 1);
}

void	SetRadioButton( DialogPtr theDialog, short theItem)
{
	ControlHandle	control;
	short			itemType;
	Rect			box;
	
	GetDItem(theDialog, theItem, &itemType, &control, &box);
	SetCtlValue( control, 1);
}
/********************************************************************************

Sound101_recordplay.c extract

 1992 Praxitel, Inc. and Iggi Monahelis

The two routines below are present unchanged from Sound101_recordplay.c published
in Machtech Magazine March 1993.  These are the necessary routines for playing 
'snd ' resource sounds.

*********************************************************************************
*********************************************************************************

PlaySND

Plays an 'snd ' resource sound using the SndStartFilePlay toolbox call.  The only 
parameter is the resource id of the sound to be played.  The sound file has allready 
been opened by the routine that called PlaySND.  We pass the id of the sound resource
to be played to the SndStartFilePlay routine.

The sound is played asynchronously.  For this reason, we set a global variable
(gDestroyChannel) to FALSE.  When the sound is done playing our callback routine is 
called (MyCallBack), which sets gDestroyChannel to TRUE.  This is our indication that
the sound is done playing and we can go ahead and close the file, and free the sound
channel.

NOTE:
The userInfo field of the sound channel structure is a convenient place to store the
current A5.  This way in the callback routine we can have access to our A5, and thus have 
access to our globals.  See the TechNotes for more info about SetCurrentA5 and SetA5.

*************************************************************************************/
void PlaySND(int tempId)
{
	gDestroyChannel = 0x0;
	gSndChan->userInfo = SetCurrentA5();
	/* since this is a sound resource, we pass its id to SndStartFilePlay */
 	SndStartFilePlay(gSndChan, 0, tempId, 20L*1024L, 0x0L,
                             0x0L, (ProcPtr) MyCallBack, TRUE);
}

/************************************************************************************

MyCallBack

When the sound is done playing, this callback routine is called.  We indicated this
routine to be called in the SndStartFilePlay call above.

This routine basically sets our global flag (gDestroyChannel) to TRUE.  This is 
our indication that the sound is done playing and we can go ahead and dispose of 
the sound channel we allocated, and close the file that contained the sound.
 
*************************************************************************************/
pascal void MyCallBack (SndChannelPtr chan)
{
	OSErr myErr;
	long  myA5;
	
	myA5 = SetA5( chan->userInfo ); 	
	gDestroyChannel = TRUE;
	myA5 = SetA5( myA5 ); 	

}
